import type { Root } from 'mdast';
import type { Plugin } from 'unified';
/**
 * Options type for the remark-remove-url-trailing-slash plugin.
 */
export type Options = {
  /**
   * If this option is `false`, trailing slashes will be removed from all URL
   * paths, including single-character `/` paths (i.e. "empty" paths).
   *
   * If this option is `true`, trailing slashes will only be removed from
   * non-relative URLs with empty paths, e.g. `https://example.com/ =>
   * https://example.com`.
   *
   * @default false
   */
  onlyConsiderHostUrls?: boolean;
};
/**
 * A remark plugin that takes a Root node as input and returns the same node
 * with all Link, Image, and Definition node `url` paths stripped of
 * trialing slashes.
 */
declare const remarkRemoveUrlTrailingSlash: Plugin<[options: Options] | void[], Root>;
export default remarkRemoveUrlTrailingSlash;